package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.List;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractVariableDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.ExpressionContext;
import org.eclipse.jdt.internal.compiler.ast.FakedTrackingVariable;
import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;
import org.eclipse.jdt.internal.compiler.parser.RecoveryScanner;

public class LocalDeclaration extends AbstractVariableDeclaration {
  public LocalVariableBinding binding;
  
  public LocalDeclaration(char[] name, int sourceStart, int sourceEnd) {
    this.name = name;
    this.sourceStart = sourceStart;
    this.sourceEnd = sourceEnd;
    this.declarationEnd = sourceEnd;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    UnconditionalFlowInfo unconditionalFlowInfo2;
    if ((flowInfo.tagBits & 0x1) == 0)
      this.bits |= 0x40000000; 
    if (this.initialization == null)
      return flowInfo; 
    this.initialization.checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
    FlowInfo preInitInfo = null;
    boolean shouldAnalyseResource = (this.binding != null && 
      flowInfo.reachMode() == 0 && 
      (currentScope.compilerOptions()).analyseResourceLeaks && 
      FakedTrackingVariable.isAnyCloseable(this.initialization.resolvedType));
    if (shouldAnalyseResource) {
      unconditionalFlowInfo2 = flowInfo.unconditionalCopy();
      FakedTrackingVariable.preConnectTrackerAcrossAssignment((ASTNode)this, this.binding, this.initialization, flowInfo);
    } 
    UnconditionalFlowInfo unconditionalFlowInfo1 = 
      this.initialization
      .analyseCode(currentScope, flowContext, flowInfo)
      .unconditionalInits();
    if (shouldAnalyseResource) {
      FakedTrackingVariable.handleResourceAssignment(currentScope, (FlowInfo)unconditionalFlowInfo2, (FlowInfo)unconditionalFlowInfo1, flowContext, (ASTNode)this, this.initialization, this.binding);
    } else {
      FakedTrackingVariable.cleanUpAfterAssignment(currentScope, 2, this.initialization);
    } 
    int nullStatus = this.initialization.nullStatus((FlowInfo)unconditionalFlowInfo1, flowContext);
    if (!unconditionalFlowInfo1.isDefinitelyAssigned(this.binding)) {
      this.bits |= 0x8;
    } else {
      this.bits &= 0xFFFFFFF7;
    } 
    unconditionalFlowInfo1.markAsDefinitelyAssigned(this.binding);
    if ((currentScope.compilerOptions()).isAnnotationBasedNullAnalysisEnabled)
      nullStatus = NullAnnotationMatching.checkAssignment(currentScope, flowContext, (VariableBinding)this.binding, (FlowInfo)unconditionalFlowInfo1, nullStatus, this.initialization, this.initialization.resolvedType); 
    if ((this.binding.type.tagBits & 0x2L) == 0L)
      unconditionalFlowInfo1.markNullStatus(this.binding, nullStatus); 
    return (FlowInfo)unconditionalFlowInfo1;
  }
  
  public void checkModifiers() {
    if ((this.modifiers & 0xFFFF & 0xFFFFFFEF) != 0)
      this.modifiers = this.modifiers & 0xFFBFFFFF | 0x800000; 
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if (this.binding.resolvedPosition != -1)
      codeStream.addVisibleLocalVariable(this.binding); 
    if ((this.bits & Integer.MIN_VALUE) == 0)
      return; 
    int pc = codeStream.position;
    if (this.initialization != null)
      if (this.binding.resolvedPosition < 0) {
        if (this.initialization.constant == Constant.NotAConstant)
          this.initialization.generateCode(currentScope, codeStream, false); 
      } else {
        this.initialization.generateCode(currentScope, codeStream, true);
        if (this.binding.type.isArrayType() && 
          this.initialization instanceof CastExpression && 
          (((CastExpression)this.initialization).innermostCastedExpression()).resolvedType == TypeBinding.NULL)
          codeStream.checkcast(this.binding.type); 
        codeStream.store(this.binding, false);
        if ((this.bits & 0x8) != 0)
          this.binding.recordInitializationStartPC(codeStream.position); 
      }  
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public int getKind() {
    return 4;
  }
  
  public void getAllAnnotationContexts(int targetType, LocalVariableBinding localVariable, List allAnnotationContexts) {
    TypeReference.AnnotationCollector collector = new TypeReference.AnnotationCollector(this, targetType, localVariable, allAnnotationContexts);
    traverseWithoutInitializer((ASTVisitor)collector, null);
  }
  
  public void getAllAnnotationContexts(int targetType, int parameterIndex, List allAnnotationContexts) {
    TypeReference.AnnotationCollector collector = new TypeReference.AnnotationCollector(this, targetType, parameterIndex, allAnnotationContexts);
    traverse((ASTVisitor)collector, null);
  }
  
  public boolean isArgument() {
    return false;
  }
  
  public boolean isReceiver() {
    return false;
  }
  
  public void resolve(BlockScope scope) {
    TypeBinding variableType = this.type.resolveType(scope, true);
    this.bits |= this.type.bits & 0x100000;
    checkModifiers();
    if (variableType != null) {
      if (variableType == TypeBinding.VOID) {
        scope.problemReporter().variableTypeCannotBeVoid(this);
        return;
      } 
      if (variableType.isArrayType() && ((ArrayBinding)variableType).leafComponentType == TypeBinding.VOID) {
        scope.problemReporter().variableTypeCannotBeVoidArray(this);
        return;
      } 
    } 
    Binding existingVariable = scope.getBinding(this.name, 3, (InvocationSite)this, false);
    if (existingVariable != null && existingVariable.isValidBinding()) {
      boolean localExists = existingVariable instanceof LocalVariableBinding;
      if (localExists && (this.bits & 0x200000) != 0 && scope.isLambdaSubscope() && this.hiddenVariableDepth == 0) {
        scope.problemReporter().lambdaRedeclaresLocal(this);
      } else if (localExists && this.hiddenVariableDepth == 0) {
        scope.problemReporter().redefineLocal(this);
      } else {
        scope.problemReporter().localVariableHiding(this, existingVariable, false);
      } 
    } 
    if ((this.modifiers & 0x10) != 0 && this.initialization == null)
      this.modifiers |= 0x4000000; 
    this.binding = new LocalVariableBinding(this, variableType, this.modifiers, false);
    scope.addLocalVariable(this.binding);
    this.binding.setConstant(Constant.NotAConstant);
    if (variableType == null) {
      if (this.initialization != null)
        this.initialization.resolveType(scope); 
      return;
    } 
    if (this.initialization != null) {
      if (this.initialization instanceof ArrayInitializer) {
        TypeBinding initializationType = this.initialization.resolveTypeExpecting(scope, variableType);
        if (initializationType != null) {
          ((ArrayInitializer)this.initialization).binding = (ArrayBinding)initializationType;
          this.initialization.computeConversion((Scope)scope, variableType, initializationType);
        } 
      } else {
        this.initialization.setExpressionContext(ExpressionContext.ASSIGNMENT_CONTEXT);
        this.initialization.setExpectedType(variableType);
        TypeBinding initializationType = this.initialization.resolveType(scope);
        if (initializationType != null) {
          if (TypeBinding.notEquals(variableType, initializationType))
            scope.compilationUnitScope().recordTypeConversion(variableType, initializationType); 
          if (this.initialization.isConstantValueOfTypeAssignableToType(initializationType, variableType) || 
            initializationType.isCompatibleWith(variableType, (Scope)scope)) {
            this.initialization.computeConversion((Scope)scope, variableType, initializationType);
            if (initializationType.needsUncheckedConversion(variableType))
              scope.problemReporter().unsafeTypeConversion(this.initialization, initializationType, variableType); 
            if (this.initialization instanceof CastExpression && (
              this.initialization.bits & 0x4000) == 0)
              CastExpression.checkNeedForAssignedCast(scope, variableType, (CastExpression)this.initialization); 
          } else if (isBoxingCompatible(initializationType, variableType, this.initialization, (Scope)scope)) {
            this.initialization.computeConversion((Scope)scope, variableType, initializationType);
            if (this.initialization instanceof CastExpression && (
              this.initialization.bits & 0x4000) == 0)
              CastExpression.checkNeedForAssignedCast(scope, variableType, (CastExpression)this.initialization); 
          } else if ((variableType.tagBits & 0x80L) == 0L) {
            scope.problemReporter().typeMismatchError(initializationType, variableType, (ASTNode)this.initialization, null);
          } 
        } 
      } 
      if (this.binding == Expression.getDirectBinding(this.initialization))
        scope.problemReporter().assignmentHasNoEffect(this, this.name); 
      this.binding.setConstant(
          this.binding.isFinal() ? 
          this.initialization.constant.castTo((variableType.id << 4) + this.initialization.constant.typeID()) : 
          Constant.NotAConstant);
    } 
    resolveAnnotations(scope, this.annotations, (Binding)this.binding, true);
    Annotation.isTypeUseCompatible(this.type, (Scope)scope, this.annotations);
    if (!scope.validateNullAnnotation(this.binding.tagBits, this.type, this.annotations))
      this.binding.tagBits &= 0xFE7FFFFFFFFFFFFFL; 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.annotations != null) {
        int annotationsLength = this.annotations.length;
        for (int i = 0; i < annotationsLength; i++)
          this.annotations[i].traverse(visitor, scope); 
      } 
      this.type.traverse(visitor, scope);
      if (this.initialization != null)
        this.initialization.traverse(visitor, scope); 
    } 
    visitor.endVisit(this, scope);
  }
  
  private void traverseWithoutInitializer(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.annotations != null) {
        int annotationsLength = this.annotations.length;
        for (int i = 0; i < annotationsLength; i++)
          this.annotations[i].traverse(visitor, scope); 
      } 
      this.type.traverse(visitor, scope);
    } 
    visitor.endVisit(this, scope);
  }
  
  public boolean isRecoveredFromLoneIdentifier() {
    return (this.name == RecoveryScanner.FAKE_IDENTIFIER && (
      this.type instanceof org.eclipse.jdt.internal.compiler.ast.SingleTypeReference || (this.type instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference && !(this.type instanceof org.eclipse.jdt.internal.compiler.ast.ArrayQualifiedTypeReference))) && this.initialization == null && !this.type.isBaseTypeReference());
  }
}
